#pragma once

#include <imgui.h>
#include <imgui_impl_glfw.h>
#include <imgui_impl_vulkan.h>

#define VULKAN_HPP_NO_CONSTRUCTORS
#include <vulkan/vulkan.hpp>

#include "../camera/camera.hpp"
#include "../structs/light_info.hpp"

class ImGuiManager {
 public:
  ImGuiManager()                                   = default;
  ImGuiManager(const ImGuiManager&)                = delete;
  ImGuiManager(ImGuiManager&&)                     = delete;
  fn operator=(const ImGuiManager&)->ImGuiManager& = delete;
  fn operator=(ImGuiManager&&)->ImGuiManager&      = delete;
  ~ImGuiManager();

  void init(
    vk::Instance            instance,
    vk::PhysicalDevice      physicalDevice,
    vk::Device              device,
    uint32_t                queueFamily,
    vk::Queue               queue,
    vk::RenderPass          renderPass,
    uint32_t                minImageCount,
    uint32_t                imageCount,
    vk::SampleCountFlagBits msaaSamples,
    GLFWwindow*             window
  );

  fn        cleanup() -> void;
  static fn newFrame() -> void;
  static fn render() -> void;
  static fn renderControls(
    float&     cameraSpeed,
    float&     fieldOfView,
    bool&      wireframeMode,
    float&     lineWidth,
    float      maxLineWidth,
    bool       wideLineSupport,
    Camera&    camera,
    LightInfo& lightSettings,
    bool&      needsPipelineRecreation
  ) -> void;
  static fn renderMemoryUsage(size_t vertexSize, size_t indexSize, size_t vertexCount, size_t indexCount)
    -> void;

 private:
  vk::UniqueDescriptorPool mDescriptorPool;
  vk::Device               mDevice      = nullptr;
  bool                     mInitialized = false;

  static fn renderControlsHelp() -> void;
  static fn renderFrameMetrics() -> void;
  static fn renderCameraInfo(const Camera& camera) -> void;
  static fn renderLightControls(LightInfo& lightSettings) -> void;
};
